extends KinematicBody

onready var ownId = self.name

######## MOVEMENT AND POSITION VARS ########

# The side faced by the player
var orientation:int = 1

# Speed values
const MAX_SPEED:float = 8.0
const MAX_SPRINT_SPEED:float = 12.0
var motion:float = 0
var isSprinting:bool = false

# Acceleration values
const ACCEL:float = 8.5
const SPRINT_ACCEL:float = 16.0
const DEACCEL:float = 16.0

# Jump and fall values
const JUMP_SPEED:float = 16.0
var isJumping:bool = false
# TODO: unreal, we may want to stick that to the physical value
const GRAVITY:float = -28.0

# Velocity updated continuously by move_and_slide
var vel:Vector3 = Vector3()

# move_and_slide need that
const FLOOR_NORMAL:Vector3 = Vector3(0,1,0)

######### ATTACKS VARS ########

# l2r
var isAttacking:bool = false
# attack type
enum {NONE=0, PRIMARY=1, SECONDARY=2}
# attacks related nodes
onready var attackTimer:Tween = $attackTween
onready var primaryHitArea:Area = $primaryHitArea
onready var secondaryHitArea:Area = $secondaryHitArea

######## FUNCTIONS ########

# called by the engine
func _physics_process(delta):
	processMovement(delta)
	broadcastMovement()

# Called from the game script to update the vars
func getPlayerInputs(movementInput, sprintInput, jumpInput, attackTypeInput):
	motion = movementInput
	orientation = movementInput if movementInput!=0 else orientation
	isSprinting = sprintInput
	isJumping = jumpInput
	if !isAttacking && attackTypeInput:
		processAttack(attackTypeInput)
	
func processMovement(delta):
	if orientation == 1:
		set_rotation_degrees(Vector3(0,0,0))
	else: 
		set_rotation_degrees(Vector3(0,180,0))
	
	if self.is_on_floor():
		if isJumping:
			vel.y = JUMP_SPEED
	
	# Consider the gravity
	vel.y += delta * GRAVITY
	
	# Speed of the player
	var target:Vector3 = Vector3(motion, 0, 0)
	
	if isSprinting:
		target *= MAX_SPRINT_SPEED
	else:
		target *= MAX_SPEED
	
	# Acceleration of the player if he is moving horizontally
	var accel:float
	var hvel:Vector3 = Vector3(vel.x,0,0)
	if target.dot(hvel)>0:
		if isSprinting:
			accel = SPRINT_ACCEL
		else:
			accel = ACCEL
	else:
		accel = DEACCEL
		
	# Interpolating speed and acceleration through time and getting it
	vel.x = hvel.linear_interpolate(target, accel * delta).x
	
	# Update the velocity after the collisions (physical engine ftw)
	vel = self.move_and_slide(vel, FLOOR_NORMAL)
	
# Send the position of the player
func broadcastMovement():
	rpc("getRemoteMovement", self.get_translation())
	
# Check and trigger the type of attack requested
func processAttack(attackType):
	
	# Turn off the other attacks input for the time being
	isAttacking = true
	
	# Use the right associated function
	match attackType:
		PRIMARY:
			primaryAttack()
		SECONDARY:
			secondaryAttack()

func primaryAttack():
	
	# Set active the hitbox of the attack, its visual feedback on server 
	primaryHitArea.set_monitoring(true)
	primaryHitArea.set_visible(true)
	var duration:float = 0.6
	var distance:float = 1.2
	
	# Prepare the translation as an interpolation over time (end position, duration and function used)
	attackTimer.interpolate_property(primaryHitArea, "translation:x", 0, distance, duration, Tween.TRANS_BACK, Tween.EASE_IN)
	attackTimer.start()
	
	# Once the movement is finished, turn off the hitbox and visual feedback
	yield(attackTimer, "tween_completed")
	isAttacking = false
	primaryHitArea.set_monitoring(false)
	primaryHitArea.set_visible(false)
	
func secondaryAttack():
	
	# Set active the hitbox of the attack, its visual feedback on server  
	secondaryHitArea.set_monitoring(true)
	secondaryHitArea.set_visible(true)
	var duration:float = 0.3
	var distance:float = 1.2
	
	# Prepare the translation as an interpolation over time (end position, duration and function used)
	attackTimer.interpolate_property(secondaryHitArea, "translation:x", 0, distance, duration, Tween.TRANS_BACK, Tween.EASE_IN)
	attackTimer.start()
	
	# Once the movement is finished, turn off the hitbox and visual feedback
	yield(attackTimer, "tween_completed")
	secondaryHitArea.set_monitoring(false)
	secondaryHitArea.set_visible(false)
	isAttacking = false
	
# Called when detected by an attack's hitbox
func hurt(damages):
	rpc_id(int(ownId), "hurt", damages);

######## SIGNALS ########

# Primary hit has landed on something
func _on_primaryHitArea_body_entered(body):
	if body.name==ownId:
		return
	
	print("body named "+body.name+" hit: primary")
	body.hurt(30)

# Secondary hit has landed on something
func _on_secondaryHitArea_body_entered(body):
	if body.name==ownId:
		return
	
	print("body named "+body.name+" hit: secondary")

